#!/usr/bin/php -q
<?php

/*
 * SNMP traffic metter by johnrembo@netzond.com
 *
 * Script will retrieve and display apropriate snmp counter(gauge) or recalculate it to "traffic"
 *
 * Script uses snmp version priority and availability map. If php snmp extension  is not available 
 * - values will be retrieved using native snmpget (much slower) otherwise it prefers php-snmp to 
 * native ucd-snmp. As far as php4 is compatible only with snmp v1, priority and awailability map 
 * will prefer snmp v1 queries to snmp v2,v3 even if one of those will be passed via SNMP_VERSION 
 * setting.
 */

if ( !isset($_SERVER['argc']) || $_SERVER['argc']<5 )
    die ("error: not enough arguments!\n\tusage: snmp.php IP SNMP-COMMUNITY OID TYPE (optional v=SNMP_VERSION[0,1,2,3] p=SNMP_PORT t=SNMP_TIMEOUT[miliseconds])\n\texample: ".$_SERVER['argv'][0]." 127.0.0.1 public IF-MIB:ifInOctets.1 1 v=2 p=161 t=500\n\tNOTES:\n\tOIDs can be numeric or textual\n\tSNMP versions: 0-native snmpget, 1,2 and 3 - apropriate php extensions\n");

//system native snmpget initialization
$snmp_binary = exec("/usr/bin/which snmpget");
if (isset($snmp_binary) && !empty($snmp_binary)) {
    $SNMP_V0=true; 
    $SNMP_V0_BINARY=$snmp_binary;
} else {
    $SNMP_V0=false;
}

//php snmp extention initialization
$snmp_status = @array_flip(get_extension_funcs("snmp"));
$SNMP_V1 = ( isset($snmp_status) && is_array($snmp_status) && isset($snmp_status['snmpget']) ) ? true : false;
$SNMP_V2 = ( isset($snmp_status) && is_array($snmp_status) && isset($snmp_status['snmp2_get']) ) ? true : false;
$SNMP_V3 = ( isset($snmp_status) && is_array($snmp_status) && isset($snmp_status['snmp3_get']) ) ? true : false;

if (!$SNMP_V0 && !$SNMP_V1 && !$SNMP_V2 && !$SNMP_V3)
    die("error: ".$_SERVER['argv'][0]." can not run on this machine!\nplease install php snmp extension or NET-SNMP module\n");

//external arguments initialization
$IP = next($_SERVER['argv']);	//IP			-query is being sent to
$COM = next($_SERVER['argv']);	//SNMP COMMUNITY	-ro community
$OID= next($_SERVER['argv']);	//SNMP OID		-textual or numeric representation
$TYPE = next($_SERVER['argv']);	//DATA TYPE 		-gauge or counter(traffic)

while ($item=next($_SERVER['argv'])) {
    $key=substr($item,0,1);
    $val=substr($item,2);
    if ($key == 'v') {
	$VERSION=$val;
    } elseif ($key =='p') {
	$PORT=$val;
    } elseif ($key =='t') {
	$TIMEOUT=$val;
    }
}

$SNMP_TIMEOUT = ( !isset($TIMEOUT) ) ? 500000 : $TIMEOUT*1000;	//microseconds (will be converted to seconds for native snmp)
$SNMP_VERSION = ( !isset($VERSION) ) ? 2 : $VERSION;		//SNMP Version (0-native snmp, 1,2,3 - apropriate php extensions)
$IP = ( !isset($PORT) ) ? "$IP:161" : "$IP:$PORT";

$REFRESH_INTERVAL=1;	//in seconds for counter(traffic) mode. snmp_traffic=(snmp_counter_old+snmp_counter_new)/refresh_interval. longer interval means better precision.
$MULTIPLIER=1;		//convert Bytes into Bits (set to 1 to disable conversion, or to 8)
$RETRIES=1;		//retry count of onsuccessful queries

if ($TYPE == 1) {	//gauge
    $response =  snmp_data($IP,$COM,$OID);
    echo $response[0];
    exit(0);
} else {		//counter
    $counter['prev'] = snmp_data($IP,$COM,$OID);
    sleep($REFRESH_INTERVAL);
    $counter['curr'] = snmp_data($IP,$COM,$OID);
    $result=round($MULTIPLIER*($counter['curr'][0] - $counter['prev'][0]) / ($counter['prev'][1]/2+$counter['curr'][1]/2+$REFRESH_INTERVAL),2)."\n";
    echo $result;
    exit(0);
}

/*
 *raw snmp query realization in preferred order
 */
function get_snmp($ip,$com,$oid) {
    global $SNMP_V0, $SNMP_V1, $SNMP_V2, $SNMP_V3, $SNMP_VERSION, $SNMP_PORT, $SNMP_TIMEOUT, $RETRIES;
    
    $V_MAP=array(
		array( array(0,$SNMP_V0),array(2,$SNMP_V2),array(1,$SNMP_V1),array(3,$SNMP_V3) ),
		array( array(1,$SNMP_V1),array(2,$SNMP_V2),array(3,$SNMP_V3),array(0,$SNMP_V0) ),
		array( array(2,$SNMP_V2),array(1,$SNMP_V1),array(3,$SNMP_V3),array(0,$SNMP_V0) ),
		array( array(3,$SNMP_V3),array(2,$SNMP_V2),array(1,$SNMP_V1),array(0,$SNMP_V0) )
		);

    foreach ($V_MAP[$SNMP_VERSION] as $details)
	if ($details[1]) {
	    $result = @call_user_func_array("snmp".$details[0]."_get",array($ip,$com,$oid,$SNMP_TIMEOUT,$RETRIES)); 
	    break;
	}
    return $result;
}

/*
 *snmp query realization registering (end - start) time
 *returns array ( 0=>counter_value, 1 => query_time )
 */
function snmp_data($ip,$com,$oid) {
    $timePrev=time() + microtime();
    
    $counter=get_snmp($ip,$com,$oid);
    
    $counter=trim(substr($counter,strrpos($counter,":")+2));

    $timeNow=time() + microtime() - $timePrev;
    $result[]=$counter;
    $result[]=$timeNow;
    
    return $result;
}

/*
 *correction of difference between different version php snmp function names
 */
function snmp1_get ($ip,$com,$oid,$timeout,$retires) {
    return @snmpget($ip,$com,$oid,$timeout,$retries);
}

/*
 *for own convenience
 */
function snmp0_get ($ip,$com,$oid,$timeout,$retires) {
    global $SNMP_V0_BINARY;
    $timeout = ($t=$timeout/1000000) < 1 ? 1 : $t;
    return @shell_exec("$SNMP_V0_BINARY -O n -t $timeout -v 2c -c $com $ip $oid");
}

?>
