<?php

//regular expressions to check parameters validity
$l=array (
	"boolean"	=>	"/^(true|false)$/",
	"digits"	=>	"/^\d+$/i",
	"x_digit"	=>	"/^[0-9]{1}$/",
	"xy_digit"	=>	"/^[0-9]{1,2}$/",
	"xyz_digit"	=>	"/^[0-9]{1,3}$/",
	"rgb_color"	=>	"/^\d{1,3},\d{1,3},\d{1,3}$/",
	"short"		=>	"/^([01][0-9][0-9]|2[0-4][0-9]|25[0-5])$/",
	"enclosers"	=>	"/^(\[|\]|\||\(|\)){1}$/",
	"string"	=>	"string",
	"denied_symbols"=>	array('#','$','%')
	);

//parameters array with predefined value ranges
$v=array(
	"data_params"	=>	array(
					"top_scale_type"	=>	"/^(data|time)$/",
					"bot_scale_type"	=>	"/^(data|time)$/",
					"time_format"		=>	$l["string"],
					"dimension_open"	=>	$l["enclosers"],
					"dimension_close"	=>	$l["enclosers"]
				    ),
				    
	"sw_params"	=>	array(
					"units"			=>	$l["string"],
					"factor"		=>	"/^(1000|1024)$/",
					"round_digits"		=>	$l["x_digit"], 
					"traffic_mode"		=>	$l["boolean"],
					"fixed_max_scale"	=>	$l["boolean"],
					"max_scal_val"		=>	$l["digits"],
					"fixed_min_scale"	=>	$l["boolean"],
					"min_scal_val"		=>	$l["digits"],
					"val_area_size"		=>	$l["x_digit"]
				    ),
				    
	"outer_params"	=>	array(
					"border_width"		=>	$l["x_digit"],
					"line_spacing"		=>	$l["x_digit"],
					"font"			=>	$l["string"],
					"font1_size"		=>	$l["xy_digit"], 
					"font2_size"		=>	$l["xy_digit"],
					"font3_size"		=>	$l["xy_digit"],
					"graph_title_top"	=>	$l["string"],
					"graph_title_bot"	=>	$l["string"],
					"axis_title_top"	=>	$l["string"],
					"axis_title_bot"	=>	$l["string"],
					"axis_title_left"	=>	$l["string"],
					"axis_title_right"	=>	$l["string"],
					"indicator"		=>	$l["boolean"],
					"refresh_interval"	=>	"/^[0-9]{3,5}$/",
					"data_source"		=>	$l["string"]
				    ),
				    
	"inner_params"	=>	array(
					"step"			=>	$l["xy_digit"],
					"horz_inner_margin"	=>	$l["xyz_digit"],
					"vert_inner_margin"	=>	$l["xyz_digit"],
					"maj_vert_grid_dens"	=>	$l["xyz_digit"],
					"min_vert_grid_dens"	=>	$l["xyz_digit"],
					"maj_horz_grid_dens"	=>	$l["xyz_digit"],
					"min_horz_grid_dens"	=>	$l["xyz_digit"],
					"bot_index_display"	=>	$l["x_digit"],
					"top_index_display"	=>	$l["x_digit"],
					"right_axis"		=>	$l["boolean"],
					"left_axis"		=>	$l["boolean"],
					"rotated_bot_axis"	=>	$l["boolean"],
					"rotated_top_axis"	=>	$l["boolean"],
					"indicator_type"	=>	"/^(solid|dash)$/",
					"indicator_width"	=>	$l["x_digit"],
					"legend_spacing"	=>	$l["xyz_digit"],
					"legend_per_line"	=>	$l["x_digit"],
					"legend_layout"		=>	"/^(static|dynamic)$/",
					"retain_axis_colors"	=>	$l["boolean"]
				    ),
	"colors"	=>	array(
					'all'	=>	$l["rgb_color"]
				    ),
	"color_map"	=>	array(
					"gr_border_start"	=>	"in_array,colors",
					"gr_border_end"		=>	"in_array,colors",
					"gr_backg_start"	=>	"in_array,colors",
					"gr_backg_end"		=>	"in_array,colors",
					"gr_title"		=>	"in_array,colors",
					"min_grid"		=>	"in_array,colors",
					"maj_grid"		=>	"in_array,colors",
					"chart_border"		=>	"in_array,colors",
					"axis_val"		=>	"in_array,colors",
					"on-chart_val"		=>	"in_array,colors",
					"indicator"		=>	"in_array,colors",
					"legend"		=>	"in_array,colors",
					"axies_labels"		=>	"in_array,colors",
                                        "chart_backg_start"     =>      "in_array,colors",
                                        "chart_backg_end"       =>      "in_array,colors",
                                        "label_rect_border"     =>      "in_array,colors"
				    ),
	"sw_string"	=>	array(
					"data_source_id"	=>	$l["x_digit"],
					"chart_type"		=>	"/^(stack-)?(line|plot|bars)$/",
					"fill_color_start"	=>	"in_array,colors",
					"fill_color_end"	=>	"in_array,colors",
					"alpha_transparnc"	=>	$l["short"],
					"smooth_edges"		=>	$l["boolean"],
					"show_details"		=>	$l["boolean"],
					"name_in_lgd"		=>	$l["string"],
					"units_in_lgd"		=>	$l["boolean"],
					"curr_val_in_lgd"	=>	$l["boolean"],
					"line_type"		=>	"/^(solid|dash)$/",
					"line_width"		=>	$l["x_digit"],
					"on_chart_values"	=>	$l["boolean"],
					"darken_contours"	=>	$l["boolean"],
					"hide_data_source"	=>	$l["boolean"]
				    )
    );

/*
 * Parameters errors descriptions
 */
function ret_err($def) {
    switch ($def) {
    case "/^(true|false)$/":
	return "only values true/false are allowed";
    case "/^\d+$/i":
	return "only digits allowed";
    case "/^[0-9]{1}$/":
	return "only one digit allowed";
    case "/^[0-9]{1,2}$/":
	return "only one or two digits allowed";
    case "/^[0-9]{1,3}$/":
	return "only one,two or three digits allowed";
    case "/^\d{1,3},\d{1,3},\d{1,3}$/":
	return "only RGB values allowed in form: [0-255],[0-255],[0-255]";
    case "/^([01][0-9][0-9]|2[0-4][0-9]|25[0-5])$/":
	return "only values from 0 to 255 allowed";
    case "/^(\[|\]|\||\(|\)){1}$/":
	return "only simple enclosers allowed: [, ], |, (, )";
    case "/^(data|time)$/":
	return "only values data/time allowed";
    case "/^(1000|1024)$/":
	return "only values 1000/1024 allowed";
    case "/^[0-9]{3,5}$/":
	return "only digits from 3 to 5 symbols allowed";
    case "/^(static|dynamic)$/":
	return "only values static/dynamic allowed";
    case "/^(stack-)?(line|plot|bars)$/":
	return "only values line/plot/bars/stack-line/stack-plot/stack-bars allowed";
    case "/^(solid|dash)$/": 
	return "only values solid/dash allowed";
    default :
	return "";
    }
}

?>
